package com.zanichelli.android.geotracker;

import android.app.Application;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.util.Patterns;

import java.util.regex.Matcher;

public class GeoTrackerApplication extends Application {
  private GeoTrackerActivity currentActivity;

  // utilizzato dalla MainActivity per mostrare
  // una o l'altra versione della finestra
  // delle impostazioni
  private boolean isFirstLaunch;

  private SharedPreferences sharedPreferences;

  // dati inseriti dall'utente all'interno della MainActivity
  private String accessCode = "";
  private String server = "";
  public final static int serverPort = 12345;

  @Override
  public void onCreate() {
    super.onCreate();

    sharedPreferences = PreferenceManager
      .getDefaultSharedPreferences(this);

    server = sharedPreferences.getString(
      getString(R.string.pref_server_key),
      ""
    );

    accessCode = sharedPreferences.getString(
      getString(R.string.pref_access_code_key),
      ""
    );

    setFirstLaunch();
  }

  public boolean isFirstLaunch() {
    return isFirstLaunch;
  }

  // all'utente viene mostrata una finestra diversa
  // se non sono stati inseriti dei dati o questi ultimi
  // non sono completi
  private void setFirstLaunch() {
    isFirstLaunch = accessCode.length() == 0 || server.length() == 0;
  }

  public String getAccessCode() {
    return accessCode;
  }

  public void setAccessCode(String accessCode) {
    // limita la lunghezza della stringa a un massimo di 32 caratteri
    this.accessCode = accessCode.substring(0,
      (32 > accessCode.length()) ? accessCode.length() : 32);

    SharedPreferences.Editor editor =
      sharedPreferences.edit();
    editor.putString(
      getString(R.string.pref_access_code_key), accessCode);
    editor.apply();

    setFirstLaunch();
  }

  public String getServer() {
    return server;
  }

  public void setServer(String server) {
    this.server = server;

    SharedPreferences.Editor editor =
      sharedPreferences.edit();
    editor.putString(
      getString(R.string.pref_server_key),
      server);
    editor.apply();

    setFirstLaunch();
  }

  // indica se l'utente ha inserito le informazioni
  // necessarie per l'invio dei dati raccolti a server remoto
  public boolean isValidHost() {
    if(accessCode.isEmpty() || server.isEmpty())
      return false;
    Matcher matcher = Patterns.IP_ADDRESS.matcher(server);

    // l'indirizzo IP non è corretto oppure non è un indirizzo IP
    if (!matcher.matches())
      return false;
    return true;
  }

  // metodo che imposta la activity passata come argomento come quella corrente
  public void setCurrentActivity(GeoTrackerActivity currentActivity) {
    this.currentActivity = currentActivity;
  }

  // metrodo che inoltra la richiesta ricevuta alla activity corrente
  public boolean hasLocationPermissions() {
    if (currentActivity == null)
      return false;

    return currentActivity.checkLocationPermission();
  }

  // metodo che inoltra il cambiamento dello stato del service
  // alla activity corrente
  public void onServiceStatusChanged() {
    currentActivity.onServiceStatusChanged();
  }
}
