package com.zanichelli.android.geotracker;

import android.location.Location;
import android.os.Bundle;
import android.widget.TextView;

import com.zanichelli.android.geotracker.position.*;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

public class InfoActivity extends GeoTrackerActivity
  implements OnReverseGeocodingListener {

  private static final int REQUEST_START_LOCATION_NAME = 0;
  private static final int REQUEST_END_LOCATION_NAME = 1;

  private Position startPos = null;
  private Position endPos = null;

  private TextView startPosName;
  private TextView endPosName;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_info);

    startPosName = (TextView) findViewById(R.id.start_pos_name);
    endPosName = (TextView) findViewById(R.id.end_pos_name);
  }

  // ogni volta che la activity viene mostrata nuovamente
  // su schermo, i dati visualizzati vengono aggiornati
  // coi valori correnti
  @Override
  public void onStart() {
    super.onStart();

    BufferedReader reader;
    String line;

    double totalDistance = 0.;
    int elapsedTime = 0;
    double meanSpeed = 0.;
    Position positionA = null;
    Position positionB;

    String[] token;
    String[] datetime;

    int hour, minute, second, day, month, year;
    Time time;
    Date date;
    Latitude lat;
    Longitude lon;

    try {
      reader = new BufferedReader(
        new FileReader(
          getApplicationContext().getFilesDir()
            + "/savedData.txt"));

      try {
        while ((line = reader.readLine()) != null) {
          token = line.split(",");
          datetime = token[0].split(":|-|/");

          hour = Integer.parseInt(datetime[0]);
          minute = Integer.parseInt(datetime[1]);
          second = Integer.parseInt(datetime[2]);
          day = Integer.parseInt(datetime[3]);
          month = Integer.parseInt(datetime[4]);
          year = Integer.parseInt(datetime[5]);

          time = new Time(hour, minute, second);
          date = new Date(day, month, year);
          lat = new Latitude(Double.parseDouble(token[1]));
          lon = new Longitude(Double.parseDouble(token[2]));

          positionB = new Position(date, time, lat, lon);

          if (positionA == null) {
            // prima posizione registrata
            startPos = positionB;
          } else {
            // altrimenti la distanza incrementa
            totalDistance += positionA.distance(positionB);
          }

          endPos = positionA = positionB;
        }
      } finally {
        reader.close();
      }
    } catch (IOException ex) {
    }

    // questo significa che le statistiche di percorso
    // sono differenti da zero
    if (startPos != endPos) {
      elapsedTime = startPos.elapsedTime(endPos);
      meanSpeed = totalDistance / (double) elapsedTime;
    }

    TextView text = (TextView) findViewById(R.id.distance);
    totalDistance = (int) (totalDistance * 100) / 100.;
    text.setText(totalDistance + " m ");

    text = (TextView) findViewById(R.id.time);
    hour = elapsedTime / 3600;
    minute = (elapsedTime % 3600) / 60;
    second = (elapsedTime % 3600) % 60;
    text.setText(intToString(hour)
      + ":"
      + intToString(minute)
      + ":"
      + intToString(second));

    text = (TextView) findViewById(R.id.speed);
    // m/s -> km/h
    meanSpeed = (int) (meanSpeed * 360) / 100.;
    text.setText(meanSpeed + " km/h ");

    if (startPos != null) {
      // di base, viene visualizzata la coppia
      // latitudine/longitudine
      Location startLocation = new Location("");

      startLocation.setLatitude(
        startPos.getLatitude().getLatitude());
      startLocation.setLongitude(
        startPos.getLongitude().getLongitude());

      startPosName.setText(startLocation.toString());

      // richiesta asincrona di traduzione della posizione
      ReverseGeocoder.requestAddressFromLocation(
        REQUEST_START_LOCATION_NAME,
        startLocation,
        this, this);

      if (startPos != endPos) {
        Location endLocation = new Location("");

        endLocation.setLatitude(
          endPos.getLatitude().getLatitude());
        endLocation.setLongitude(
          endPos.getLongitude().getLongitude());

        endPosName.setText(endLocation.toString());

        // richiesta asincrona di traduzione della posizione
        ReverseGeocoder.requestAddressFromLocation(
          REQUEST_END_LOCATION_NAME,
          endLocation,
          this, this);
      }
    }
  }

  // da valore intero a stringa di due caratteri
  private String intToString(int val) {
    String string;
    string = Integer.toString(val);
    int len = string.length();
    if (len < 2)
      string = "0" + string;
    return string;
  }

  @Override
  public void onReverseGeocodingResult(
    int requestCode, String result) {

    if (result == null)
      return; // nessuna operazione da eseguire

    switch (requestCode) {
      case REQUEST_START_LOCATION_NAME:
        startPosName.setText(result);
        if (startPos == endPos)
          endPosName.setText(result);
        break;
      case REQUEST_END_LOCATION_NAME:
        endPosName.setText(result);
        break;
    }
  }
}
