package com.zanichelli.android.geotracker;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class LocationService extends Service {
  private static boolean RUN = false;
  private static boolean pendingRUN = false;

  private GeoTrackerApplication app;

  public static boolean isRunning() {
    return RUN;
  }

  public static boolean hasPendingRequest() {
    return pendingRUN;
  }

  public static void cancelPendingrequest() {
    pendingRUN = false;
  }

  public LocationService() {
  }

  private LocationListener listener = new LocationListener() {
    @Override
    public void onLocationChanged(Location location) {
      updateLocationData(location);
    }

    @Override
    public void onStatusChanged(String s, int i, Bundle bundle) {
    }

    @Override
    public void onProviderEnabled(String s) {
    }

    @Override
    public void onProviderDisabled(String s) {
    }
  };

  @Override
  public IBinder onBind(Intent intent) {
    return null;
  }

  @Override
  public void onCreate() {
    super.onCreate();

    if (RUN)
      return;

    app = (GeoTrackerApplication) getApplication();
    if (!app.hasLocationPermissions()) {
      // si registra come pendente l'esecuzione corrente
      pendingRUN = true;
      this.stopSelf();
      return;
    }

    // il servizio viene sicuramente eseguito
    pendingRUN = false;
    RUN = true;

    // eliminazione del file precedente
    File file = new File(
      getApplicationContext().getFilesDir()
        + "/" +
        "savedData.txt");
    file.delete();

    // collegamento con il provider dei dati GPS
    LocationManager manager =
      (LocationManager) getSystemService(LOCATION_SERVICE);
    try {
      manager.requestLocationUpdates(
        LocationManager.GPS_PROVIDER, // dati da GPS
        1000, // periodo (millisecondi)
        5, // distanza (metri)
        listener // listener degli aggiornamenti
      );
    } catch (SecurityException exeption) {
      // non è possibile che l'operazione indicata
      // causi una SecurityException, poiché la APP
      // controlla all'inizio dell'esecuzione del
      // service se possiede i permessi richiesti
    }

    app.onServiceStatusChanged();
  }

  @Override
  public void onDestroy() {
    super.onDestroy();

    // se l'esecuzione corrente è marcata come
    // pendente, non è deve essere eseguito
    // il resto del corpo
    if (pendingRUN)
      return;

    // arresto il servizio
    RUN = false;

    app.onServiceStatusChanged();

    // terminazione della ricezione di aggiornamenti
    LocationManager locationManager =
      (LocationManager) getSystemService(LOCATION_SERVICE);
    try {
      locationManager.removeUpdates(listener);
    } catch (SecurityException exeption) {

    }
  }

  // metodo invocato per ogni aggiornamento ricevuto
  private void updateLocationData(Location location) {
    String line = "";
    java.util.Date timestamp =
      new java.util.Date(location.getTime());
    line += Integer.toString(timestamp.getHours()) + ":";
    line += Integer.toString(timestamp.getMinutes()) + ":";
    line += Integer.toString(timestamp.getSeconds()) + "-";
    line += Integer.toString(timestamp.getDate()) + "/";
    line += Integer.toString(timestamp.getMonth() + 1) + "/";
    line += Integer.toString(timestamp.getYear() + 1900) + ",";

    double latitude = location.getLatitude();
    line += Double.toString(latitude) + ",";

    double longitude = location.getLongitude();
    line += Double.toString(longitude) + ",";

    if (location.hasAltitude()) {
      double altitude = location.getAltitude();
      line += Double.toString(altitude) + ",";
    } else
      line += " " + ",";

    if (location.hasSpeed()) {
      float speed = location.getSpeed();
      line += Float.toString(speed) + "\r\n";
    } else
      line += " " + "\r\n";

    // scrittura della stringa su file
    try {
      // apertura del file in append
      FileOutputStream file = openFileOutput("savedData.txt",
        Context.MODE_APPEND);
      file.write(line.getBytes());
      file.flush();
      file.close();
    } catch (FileNotFoundException exeption) {
    } catch (IOException exeption) {
    }

    // invio dei dati al server

    // controllo della validità delle credenziali
    // e dell'indirizzo del server
    if (!app.isValidHost())
      return;

    try {
      byte[] message = new byte[44];

      byte[] buffer = app.getAccessCode().getBytes("UTF-8");

      // copia dei 32 caratteri del codice di accesso nel messaggio
      for (int i = 0; i < 32; ++i)
        if (i < buffer.length)
          message[i] = buffer[i];
        else
          message[i] = 0;

      // wrapper per l'inserimento di valori interi
      ByteBuffer data;
      data = ByteBuffer.wrap(message, 32, 12);

      data.order(ByteOrder.LITTLE_ENDIAN);

      // tempo in secondi da 1 Gennaio 1970
      int time = (int) (location.getTime() / 1000);
      data.putInt(time);

      data.putFloat((float) (latitude));
      data.putFloat((float) (longitude));

      UDPThread thread = new UDPThread(message,
        app.getServer(),
        GeoTrackerApplication.serverPort);

      // lancio del thread
      thread.start();

    } catch (UnsupportedEncodingException exeption) {
    }
  }
}