package com.zanichelli.android.geotracker;

import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends GeoTrackerActivity
  implements View.OnClickListener {

  private Button startJourney;
  private Button endJourney;
  private Button showMap;
  private Button showInfo;
  private Button showSetting;

  private AlertDialog error;
  private AlertDialog dataDialog;
  private AlertDialog.Builder dataDialogBuilder;

  private boolean isFirstAlert = true;

  private GeoTrackerApplication app;

  private DialogInterface.OnClickListener positiveListener =
    new DialogInterface.OnClickListener() {
      @Override
      public void onClick(DialogInterface dialogInt, int i) {
        if (app.isFirstLaunch())
          Toast.makeText(MainActivity.this,
            R.string.toast_first,
            Toast.LENGTH_LONG)
            .show();
        else
          Toast.makeText(MainActivity.this,
            R.string.toast_message,
            Toast.LENGTH_LONG)
            .show();

        Dialog dialog = (Dialog) dialogInt;
        EditText edit =
          (EditText) dialog.findViewById(R.id.access_code);

        app.setAccessCode(edit.getText().toString());

        edit = (EditText) dialog.findViewById(R.id.server);
        app.setServer(edit.getText().toString());

        changeDataDialog();
      }
    };

  private DialogInterface.OnClickListener neutralListener =
    new DialogInterface.OnClickListener() {
      @Override
      public void onClick(DialogInterface dialogInt, int i) {
        app.setAccessCode("");
        app.setServer("");

        Toast.makeText(MainActivity.this,
          R.string.toast_erase,
          Toast.LENGTH_LONG)
          .show();

        changeDataDialog();
      }
    };

  // aggiorna lo stato dei bottoni di start e stop
  private void updateUI() {
    if (LocationService.isRunning()) {
      startJourney.setEnabled(false);
      endJourney.setEnabled(true);
    } else {
      startJourney.setEnabled(true);
      endJourney.setEnabled(false);
    }
  }

  private void changeDataDialog() {
    if (isFirstAlert && !app.isFirstLaunch()) {
      dataDialogBuilder
        .setPositiveButton(R.string.btn_pos_default,
          positiveListener)
        .setNegativeButton(R.string.btn_neg_default,
          null)
        .setNeutralButton(R.string.btn_neutral,
          neutralListener);
    }

    if (!isFirstAlert && app.isFirstLaunch())
      dataDialogBuilder
        .setPositiveButton(R.string.btn_pos_first,
          positiveListener)
        .setNegativeButton(R.string.btn_neg_first,
          null)
        .setNeutralButton("", null);

    dataDialog = dataDialogBuilder.create();

    isFirstAlert = app.isFirstLaunch();
  }

  // costruisce la finestra mostrata automaticamente per
  // chiedere i dati all'utente e mantenere le impostazioni
  private void buildDataAlert() {
    dataDialogBuilder = new AlertDialog.Builder(this);

    dataDialogBuilder.setTitle(R.string.settings_title)
      .setCancelable(true)
      .setView(R.layout.form_layout)
      .setPositiveButton(R.string.btn_pos_first,
        positiveListener)
      .setNegativeButton(R.string.btn_neg_first,
        null)
      .setNeutralButton("", null);

    changeDataDialog();

    if (app.isFirstLaunch()) {
      showDataDialog();
    }
  }

  private void showDataDialog() {
    dataDialog.show();

    TextView message =
      (TextView) dataDialog.findViewById(R.id.message);

    if (app.isFirstLaunch())
      message.setText(R.string.first_message);
    else
      message.setText(R.string.default_message);

    // imposta i valori delle view dela finestra
    EditText edit = (EditText)
      dataDialog.findViewById(R.id.access_code);
    edit.setText(app.getAccessCode());

    edit = (EditText) dataDialog.findViewById(R.id.server);
    edit.setText(app.getServer());
  }

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);

    startJourney = (Button) findViewById(R.id.start);
    startJourney.setOnClickListener(this);
    endJourney = (Button) findViewById(R.id.stop);
    endJourney.setOnClickListener(this);
    showMap = (Button) findViewById(R.id.map);
    showMap.setOnClickListener(this);
    showInfo = (Button) findViewById(R.id.info);
    showInfo.setOnClickListener(this);
    showSetting = (Button) findViewById(R.id.settings);
    showSetting.setOnClickListener(this);

    error = new AlertDialog.Builder(this)
      .setTitle(getString(R.string.alert_title))
      .setMessage(R.string.alert_message)
      .setCancelable(true)
      .setPositiveButton(R.string.alert_positive, null)
      .create();

    app = (GeoTrackerApplication) getApplication();

    updateUI();

    buildDataAlert();
  }

  @Override
  public void onClick(View widget) {
    switch (widget.getId()) {
      case R.id.start:
        if (!LocationService.isRunning()) {
          startService(
            new Intent(this, LocationService.class));
        }
        break;
      case R.id.stop:
        if (LocationService.isRunning()) {
          stopService(new Intent(this, LocationService.class));
        }
        break;
      case R.id.map:
        Intent maplaunch = new Intent(MainActivity.this,
          MapsActivity.class);
        startActivity(maplaunch);
        break;
      case R.id.info:
        Intent infolaunch = new Intent(MainActivity.this,
          InfoActivity.class);
        startActivity(infolaunch);
        break;
      case R.id.settings:
        showDataDialog();
        break;
    }
  }

  @Override
  public void onServiceStatusChanged() {
    super.onServiceStatusChanged();
    updateUI();
  }

  // se l'utente non concede il permesso,
  // mostra una finestra che motiva la richiesta
  // effettuata, in modo da indurre in seguito
  // l'utente a concedere il permesso
  @Override
  public void onPermissionDenied(int requestCode) {
    super.onPermissionDenied(requestCode);
    error.show();
  }
}