package com.zanichelli.android.geotracker;

import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.AsyncTask;

import java.io.IOException;
import java.util.List;

public class ReverseGeocoder {

  // metodo statico per avviare una richiesta asincrona
  static void requestAddressFromLocation(int requestCode, Location location,
                                         OnReverseGeocodingListener listener, Context context) {

    ReverseGeocoderTask task = new ReverseGeocoderTask(context, requestCode, listener);
    task.execute(location);
  }
}

class ReverseGeocoderTask extends AsyncTask<Location, Void, String> {

  private Context context;
  private int requestCode;
  private OnReverseGeocodingListener listener;

  public ReverseGeocoderTask(
    Context context,
    int requestCode,
    OnReverseGeocodingListener listener) {

    super();
    this.context = context;
    this.requestCode = requestCode;
    this.listener = listener;
  }

  @Override
  protected String doInBackground(Location... params) {
    Geocoder geocoder = new Geocoder(context);
    double latitude = params[0].getLatitude();
    double longitude = params[0].getLongitude();

    List<Address> addresses;
    String addressText = null;

    try {
      // richiesta alla classe di Google per il reverse greocoding
      addresses = geocoder.getFromLocation(latitude,
        longitude, 1);

      if (addresses != null && addresses.size() > 0) {
        // il primo e unico indirizzo restituito
        // è la traduzione cercata
        StringBuilder builder = new StringBuilder();

        Address address = addresses.get(0);
        for (int i = 0; i < address.getMaxAddressLineIndex(); i++) {
          builder.append(address.getAddressLine(i));
          if(i+1 < address.getMaxAddressLineIndex())
            builder.append(", ");
        }

        addressText = builder.toString();

        // se c'è stato un errore, il risultato deve essere null
        if(addressText.length() < 1)
          addressText = null;
      }
    } catch (IOException e) {

    }
    return addressText;
  }

  @Override
  protected void onPostExecute(String result) {
    listener.onReverseGeocodingResult(requestCode, result);
  }
}